/*

    TiMidity++ -- MIDI to WAVE converter and player
    Copyright (C) 1999 Masanao Izumo <mo@goice.co.jp>
    Copyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef ___MBLOCK_H_
#define ___MBLOCK_H_


/* Memory block for decreasing malloc
 *
 * +------+    +------+             +-------+
 * |BLOCK1|--->|BLOCK2|---> ... --->|BLOCK N|---> NULL
 * +------+    +------+             +-------+
 *
 *
 * BLOCK:
 * +-----------------------+
 * | memory 1              |
 * |                       |
 * +-----------------------+
 * | memory 2              |
 * +-----------------------+
 * | memory 3              |
 * |                       |
 * |                       |
 * +-----------------------+
 * | unused ...            |
 * +-----------------------+
 */


typedef struct _MBlockNode
{
    size_t block_size;
    size_t offset;

    int alloc_type;
#define MBLOCK_PAGE	1	/* for small memory */
#define MBLOCK_KALLOC	2	/* for small memory */
#define MBLOCK_VALLOC	3	/* for large memory */

    struct _MBlockNode *next;
    char buffer[1];
} MBlockNode;

typedef MBlockNode *MBlock;

/* #define BASE_MBLOCK_SIZE 4000 */
#define BASE_MBLOCK_SIZE (PAGE_SIZE - sizeof(struct _MBlockNode) - 32)

extern void mblock_init(MBlock *mblock);
extern void *mblock_alloc(MBlock *mblock, size_t nbytes);
extern void mblock_reuse(MBlock *mblock);
extern int mblock_free(MBlock *mblock);
extern char *mblock_strdup(MBlock *mblock, const char *str);
extern int mblock_free_all(void);

#endif /* ___MBLOCK_H_ */
